@@ -16,8 +16,10 @@ extern Dimension keypad_qheight();
 extern Dimension min_keypad_width();
 extern void keypad_first_up();
 extern Widget keypad_init();
+extern void keypad_move();
 extern void keypad_popup_init();
 extern Dimension keypad_qheight();
 extern void keypad_set_keymap();
 extern void keypad_set_temp_keymap();
 extern void keypad_shift();
+extern void ReparentNotify_action();
