@@ -728,7 +728,8 @@ Boolean is_login;
 		sms->is_login = True;
 	} else
 		sms->state = SS_INCOMPLETE;
-	sms_continue();
+	if (sms_depth == 1)
+		sms_continue();
 }
 
 /* Push a string on the stack. */
@@ -1814,6 +1815,28 @@ Cardinal *num_params;
 	++children;
 }
 
+/* "Macro" action, explicitly invokes a named macro. */
+/*ARGSUSED*/
+void
+Macro_action(w, event, params, num_params)
+Widget w;
+XEvent *event;
+String *params;
+Cardinal *num_params;
+{
+	struct macro_def *m;
+
+	if (check_usage(Script_action, *num_params, 1, 1) < 0)
+		return;
+	for (m = macro_defs; m != (struct macro_def *)NULL; m = m->next) {
+		if (!strcmp(m->name, params[0])) {
+			push_macro(m->action, False);
+			return;
+		}
+	}
+	popup_an_error("no such macro: '%s'", params[0]);
+}
+
 /* Abort all running scripts. */
 void
 abort_script()
