@@ -221,12 +221,9 @@ Cardinal *num_params;
 	/* Find the Confirm or Okay button */
 	w2 = XtNameToWidget(XtParent(w), ObjConfirmButton);
 	if (w2 == NULL)
-		w2 = XtNameToWidget(XtParent(w), ObjOkayButton);
-	if (w2 == NULL)
-		w2 = XtNameToWidget(w, ObjOkayButton);
+		w2 = XtNameToWidget(w, ObjConfirmButton);
 	if (w2 == NULL) {
-		xs_warning("confirm: cannot find %s or %s", ObjConfirmButton,
-		    ObjOkayButton);
+		xs_warning("confirm: cannot find %s", ObjConfirmButton);
 		return;
 	}
 
@@ -434,7 +431,7 @@ error_popup_init()
 	/* Create a dialog in the popup */
 
 	error_form = XtVaCreateManagedWidget(
-	    "errorDialog", dialogWidgetClass, error_shell,
+	    "dialog", dialogWidgetClass, error_shell,
 	    NULL);
 	XtVaSetValues(XtNameToWidget(error_form, XtNlabel),
 	    XtNlabel, "first line\nsecond line",
@@ -443,7 +440,7 @@ error_popup_init()
 	/* Add "OK" button to the dialog */
 
 	w = XtVaCreateManagedWidget(
-	    ObjOkayButton, commandWidgetClass, error_form,
+	    ObjConfirmButton, commandWidgetClass, error_form,
 	    NULL);
 	XtAddCallback(w, XtNcallback, saw_error, 0);
 
@@ -577,7 +574,7 @@ info_popup_init()
 	/* Create a dialog in the popup */
 
 	info_form = XtVaCreateManagedWidget(
-	    "infoDialog", dialogWidgetClass, info_shell,
+	    "dialog", dialogWidgetClass, info_shell,
 	    NULL);
 	XtVaSetValues(XtNameToWidget(info_form, XtNlabel),
 	    XtNlabel, "first line\nsecond line",
@@ -586,7 +583,7 @@ info_popup_init()
 	/* Add "OK" button to the dialog */
 
 	w = XtVaCreateManagedWidget(
-	    ObjOkayButton, commandWidgetClass, info_form,
+	    ObjConfirmButton, commandWidgetClass, info_form,
 	    NULL);
 	XtAddCallback(w, XtNcallback, saw_info, 0);
 
