@@ -2830,7 +2830,7 @@ char *ef;
 {
 	unsigned long svalue, svalue2;
 	extern Atom a_3270;
-	extern Atom a_registry, a_iso8859;
+	extern Atom a_registry, a_iso8859, a_ISO8859;
 	extern Atom a_encoding, a_1;
 
 	if (efontname)
@@ -2847,7 +2847,7 @@ char *ef;
 		nss.latin1_font =
 		    nss.efontinfo->max_char_or_byte2 > 127 &&
 		    XGetFontProperty(f, a_registry, &svalue) &&
-		    svalue == a_iso8859 &&
+		    (svalue == a_iso8859  || svalue == a_ISO8859) &&
 		    XGetFontProperty(f, a_encoding, &svalue2) &&
 		    svalue2 == a_1;
 		nss.xfmap = nss.latin1_font ? cg2asc : cg2asc7;
