@@ -52,6 +52,10 @@
 #define TELOPT_NAWS	31
 #endif /*]*/
 
+#if !defined(TELOPT_TN3270E) /*[*/
+#define TELOPT_TN3270E	40
+#endif /*]*/
+
 #define BUFSZ		4096
 #define TRACELINE	72
 
@@ -124,6 +128,8 @@ static void     net_interrupt();
 static int      non_blocking();
 static void     net_connected();
 static void     trace_str();
+static char    *cmd();
+static char    *opt();
 
 /* telnet states */
 #define TNS_DATA	0	/* receiving data */
@@ -145,22 +151,6 @@ static unsigned char	will_opt[]	= {
 static unsigned char	wont_opt[]	= { 
 	IAC, WONT, '_' };
 
-#if defined(TELCMD) && defined(TELCMD_OK)
-#define cmd(c)	(TELCMD_OK(c) ? TELCMD(c) : nnn((int)c))
-#define ccmd(c)	cmd(c)
-#else
-#define ccmd(c)	(telcmds[(c) - SE])
-#define cmd(c)	(((c) >= SE && (c) <= IAC) ? ccmd(c) : nnn((int)c))
-#endif
-
-#if defined(TELOPT) && defined(TELOPT_OK)
-#define opt(c)	(TELOPT_OK(c) ? TELOPT(c) : nnn((int)c))
-#define copt(c)	opt(c)
-#else
-#define copt(c)	(telopts[c])
-#define opt(c)	(((c) <= TELOPT_EOR) ? copt(c) : nnn((int)c))
-#endif
-
 char *telquals[2] = { "IS", "SEND" };
 
 
@@ -473,11 +463,11 @@ int n;
 
 	n %= 256 * 256;
 	if ((n / 256) == IAC)
-		*buf++ = IAC;
+		*(unsigned char *)buf++ = IAC;
 	*buf++ = (n / 256);
 	n %= 256;
 	if (n == IAC)
-		*buf++ = IAC;
+		*(unsigned char *)buf++ = IAC;
 	*buf++ = n;
 	return buf - b0;
 }
@@ -499,8 +489,8 @@ send_naws()
 	(void) sprintf(naws_msg + naws_len, "%c%c", IAC, SE);
 	naws_len += 2;
 	net_rawout((unsigned char *)naws_msg, naws_len);
-	(void) sprintf(trace_msg, "SENT %s NAWS %d %d %s\n", ccmd(SB), maxCOLS,
-	    maxROWS, ccmd(SE));
+	(void) sprintf(trace_msg, "SENT %s NAWS %d %d %s\n", cmd(SB), maxCOLS,
+	    maxROWS, cmd(SE));
 	trace_str(trace_msg);
 }
 
@@ -635,7 +625,7 @@ unsigned char c;
 				do_opt[2] = c;
 				net_rawout(do_opt, sizeof(do_opt));
 				(void) sprintf(trace_msg, "SENT %s %s\n",
-				    ccmd(DO), opt(c));
+				    cmd(DO), opt(c));
 				trace_str(trace_msg);
 			}
 			check_in3270();
@@ -644,7 +634,7 @@ unsigned char c;
 			dont_opt[2] = c;
 			net_rawout(dont_opt, sizeof(dont_opt));
 			(void) sprintf(trace_msg, "SENT %s %s\n",
-			    ccmd(DONT), opt(c));
+			    cmd(DONT), opt(c));
 			trace_str(trace_msg);
 			break;
 		}
@@ -662,7 +652,7 @@ unsigned char c;
 				dont_opt[2] = c;
 				net_rawout(dont_opt, sizeof(dont_opt));
 				(void) sprintf(trace_msg, "SENT %s %s\n",
-				    ccmd(DONT), opt(c));
+				    cmd(DONT), opt(c));
 				trace_str(trace_msg);
 			}
 			/* fall through */
@@ -683,12 +673,13 @@ unsigned char c;
 		    case TELOPT_TTYPE:
 		    case TELOPT_SGA:
 		    case TELOPT_NAWS:
+		    case TELOPT_TM:
 			if (!myopts[c]) {
 				myopts[c] = 1;
 				will_opt[2] = c;
 				net_rawout(will_opt, sizeof(will_opt));
 				(void) sprintf(trace_msg, "SENT %s %s\n",
-				    ccmd(WILL), opt(c));
+				    cmd(WILL), opt(c));
 				trace_str(trace_msg);
 			}
 			check_in3270();
@@ -699,7 +690,7 @@ unsigned char c;
 			wont_opt[2] = c;
 			net_rawout(wont_opt, sizeof(wont_opt));
 			(void) sprintf(trace_msg, "SENT %s %s\n",
-			    ccmd(WONT), opt(c));
+			    cmd(WONT), opt(c));
 			trace_str(trace_msg);
 			break;
 		}
@@ -715,7 +706,7 @@ unsigned char c;
 			wont_opt[2] = c;
 			net_rawout(wont_opt, sizeof(wont_opt));
 			(void) sprintf(trace_msg, "SENT %s %s\n",
-			    ccmd(WONT), opt(c));
+			    cmd(WONT), opt(c));
 			trace_str(trace_msg);
 			/* fall through */
 		    default:
@@ -751,8 +742,8 @@ unsigned char c;
 
 				(void) sprintf(trace_msg,
 				    "SENT %s %s %s %s %s\n",
-				    ccmd(SB), copt(TELOPT_TTYPE),
-				    telquals[TELQUAL_IS], termtype, ccmd(SE));
+				    cmd(SB), opt(TELOPT_TTYPE),
+				    telquals[TELQUAL_IS], termtype, cmd(SE));
 				trace_str(trace_msg);
 				check_in3270();
 			}
@@ -1271,6 +1262,50 @@ int	c;
 	return buf;
 }
 
+#if !defined(TELCMD) || !defined(TELCMD_OK) /*[*/
+#undef TELCMD
+#define TELCMD(x)	telcmds[(x) - SE]
+#undef TELCMD_OK
+#define TELCMD_OK(x)	((x) >= SE && (x) <= IAC)
+#endif /*]*/
+
+/*
+ * cmd
+ *	Expands a TELNET command into a character string.
+ */
+static char *
+cmd(c)
+unsigned char c;
+{
+	if (TELCMD_OK(c))
+		return TELCMD(c);
+	else
+		return nnn((int)c);
+}
+
+#if !defined(TELOPT) || !defined(TELOPT_OK) /*[*/
+#undef TELOPT
+#define TELOPT(x)	telopts[(x)]
+#undef TELOPT_OK
+#define TELOPT_OK(x)	((x) >= TELOPT_BINARY && (x) <= TELOPT_EOR)
+#endif /*]*/
+
+/*
+ * opt
+ *	Expands a TELNET option into a character string.
+ */
+static char *
+opt(c)
+unsigned char c;
+{
+	if (TELOPT_OK(c))
+		return TELOPT(c);
+	else if (c == TELOPT_TN3270E)
+		return "TN3270E";
+	else
+		return nnn((int)c);
+}
+
 
 #define LINEDUMP_MAX	32
 
@@ -1436,14 +1471,14 @@ net_linemode()
 		dont_opt[2] = TELOPT_ECHO;
 		net_rawout(dont_opt, sizeof(dont_opt));
 		(void) sprintf(trace_msg, "SENT %s %s\n",
-		    ccmd(DONT), copt(TELOPT_ECHO));
+		    cmd(DONT), opt(TELOPT_ECHO));
 		trace_str(trace_msg);
 	}
 	if (hisopts[TELOPT_SGA]) {
 		dont_opt[2] = TELOPT_SGA;
 		net_rawout(dont_opt, sizeof(dont_opt));
 		(void) sprintf(trace_msg, "SENT %s %s\n",
-		    ccmd(DONT), copt(TELOPT_SGA));
+		    cmd(DONT), opt(TELOPT_SGA));
 		trace_str(trace_msg);
 	}
 }
@@ -1456,15 +1491,15 @@ net_charmode()
 	if (!hisopts[TELOPT_ECHO]) {
 		do_opt[2] = TELOPT_ECHO;
 		net_rawout(do_opt, sizeof(do_opt));
-		(void) sprintf(trace_msg, "SENT %s %s\n", ccmd(DO),
-		    copt(TELOPT_ECHO));
+		(void) sprintf(trace_msg, "SENT %s %s\n", cmd(DO),
+		    opt(TELOPT_ECHO));
 		trace_str(trace_msg);
 	}
 	if (!hisopts[TELOPT_SGA]) {
 		do_opt[2] = TELOPT_SGA;
 		net_rawout(do_opt, sizeof(do_opt));
-		(void) sprintf(trace_msg, "SENT %s %s\n", ccmd(DO),
-		    copt(TELOPT_SGA));
+		(void) sprintf(trace_msg, "SENT %s %s\n", cmd(DO),
+		    opt(TELOPT_SGA));
 		trace_str(trace_msg);
 	}
 }
